// (C) 2007-2024 GoodData Corporation
import React from "react";
import { IntlProvider, createIntl } from "react-intl";
import { messagesMap } from "./messagesMap.js";
import { DefaultLocale, isLocale } from "./Locale.js";
import { wrapDisplayName } from "../react/wrapDisplayName.js";
/**
 * @internal
 */
export function createIntlMock(customMessages = {}, locale = "en-US") {
    return createIntl({
        locale,
        messages: {
            ...resolveLocaleDefaultMessages(locale, messagesMap),
            ...customMessages,
        },
    });
}
/**
 * @internal
 */
export function withIntl(WrappedComponent, customLocale, customMessages) {
    class WithIntl extends React.Component {
        render() {
            const locale = customLocale ? customLocale : DefaultLocale;
            const messages = customMessages
                ? customMessages
                : resolveLocaleDefaultMessages(locale, messagesMap);
            return (React.createElement(IntlProvider, { locale: locale, messages: messages },
                React.createElement(WrappedComponent, { ...this.props })));
        }
    }
    return wrapDisplayName("withIntl", WrappedComponent)(WithIntl);
}
/**
 * Resolves parameter into {@link ILocale} or {@link DefaultLocale}.
 *
 * @param locale - value of the locale to check for support
 *
 * @internal
 */
export const resolveLocale = (locale) => {
    return isLocale(locale) ? locale : DefaultLocale;
};
/**
 * This method try to resolve translations for the given locale.
 * If the locale is not found or is invalid, it will return the translations for default locale.
 * @param locale - input locale
 * @param messagesMap - map of messages
 * @returns resolved messages
 *
 * @internal
 */
export const resolveLocaleDefaultMessages = (locale, messagesMap) => {
    const strings = messagesMap[locale];
    if (!strings) {
        console.warn(`Missing locale strings for ${locale}, using ${DefaultLocale}`);
        return messagesMap[DefaultLocale];
    }
    return strings;
};
/**
 * Returns a string meant to represent a header with an empty value.
 * @param intl - the source of i18n strings
 * @internal
 */
export function emptyHeaderTitleFromIntl(intl) {
    return `(${intl.formatMessage({ id: "visualization.emptyValue" })})`;
}
/**
 * Returns a string meant to represent the total colum when it is empty.
 * @param intl - the source of i18n strings
 * @internal
 */
export function totalColumnTitleFromIntl(intl) {
    return intl.formatMessage({ id: "visualization.waterfall.total" });
}
/**
 * Returns a string meant to represent a cluster group.
 * @param intl - the source of i18n strings
 * @internal
 */
export function clusterTitleFromIntl(intl) {
    return intl.formatMessage({ id: "visualization.cluster" });
}
//# sourceMappingURL=intlUtils.js.map